﻿// $Id: PercentBarRenderer.cs 25 2008-04-20 14:51:40Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Drawing;
using System.Globalization;
using BrightIdeasSoftware;

namespace OSD.Program
{
    internal class PercentBarRenderer : BarRenderer
    {
        /// <summary>
        /// Make a BarRenderer
        /// </summary>
        public PercentBarRenderer() : base() {}

        /// <summary>
        /// Make a BarRenderer for the given range of data values
        /// </summary>
        public PercentBarRenderer(int minimum, int maximum) : base(minimum, maximum) {}

        /// <summary>
        /// Make a BarRenderer using a custom bar scheme
        /// </summary>
        public PercentBarRenderer(Pen aPen, Brush aBrush) : base(aPen, aBrush) {}

        /// <summary>
        /// Make a BarRenderer using a custom bar scheme
        /// </summary>
        public PercentBarRenderer(int minimum, int maximum, Pen aPen, Brush aBrush)
            : base(minimum, maximum, aPen, aBrush) {}

        /// <summary>
        /// Make a BarRenderer that uses a horizontal gradient
        /// </summary>
        public PercentBarRenderer(Pen aPen, Color start, Color end) : base(aPen, start, end) {}

        /// <summary>
        /// Make a BarRenderer that uses a horizontal gradient
        /// </summary>
        public PercentBarRenderer(int minimum, int maximum, Pen aPen, Color start, Color end)
            : base(minimum, maximum, aPen, start, end) {}

        /// <summary>
        /// Draw our aspect
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        public override void Render(Graphics g, Rectangle r)
        {
            base.Render(g, r);

            Rectangle frameRect = Rectangle.Inflate(r, 0 - Padding, 0 - Padding);
            frameRect.Width = Math.Min(frameRect.Width, MaximumWidth);
            frameRect.Height = Math.Min(frameRect.Height, MaximumHeight);
            frameRect = AlignRectangle(r, frameRect);

            // Convert our aspect to a numeric value
            // CONSIDER: Is this the best way to do this?
            if (!(Aspect is IConvertible))
                return;
            double aspectValue = ((IConvertible) Aspect).ToDouble(NumberFormatInfo.InvariantInfo);

            StringFormat sf = new StringFormat();
            sf.Alignment = StringAlignment.Center;
            sf.LineAlignment = StringAlignment.Center;

            if (aspectValue >= 0)
                g.DrawString(aspectValue.ToString("N0") + " %", SystemFonts.DialogFont,
                             new SolidBrush(SystemColors.WindowText), frameRect, sf);
        }
    }
}