﻿// $Id: Util.cs 25 2008-04-20 14:51:40Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System.Text.RegularExpressions;

namespace OSD.PluginBase
{
    ///<summary>
    /// Pomocna trida 
    ///</summary>
    public class Util
    {
        /// <summary>
        /// Provede regularni vyraz na zadanem obsahu a vrati true, zda vyrazu vyhovuje zadany pattern. Pokud regularni vyraz obsahuje capture groups, jsou vraceny v parametru groups.
        /// </summary>
        /// <param name="pattern">regularni vyraz</param>
        /// <param name="content">obsah, na kterem se vykona regularni vyraz</param>
        /// <param name="groups">nalezene capture groups</param>
        /// <returns>true, pokud v obsahu byl nalezen zadany pattern</returns>
        public static bool RegexMatch(string pattern, string content, out GroupCollection groups)
        {
            Regex r = new Regex(pattern);
            Match m = r.Match(content);
            if (!m.Success)
            {
                groups = null;
                return false;
            }

            groups = m.Groups;
            return true;
        }

        /// <summary>
        /// Provede regularni vyraz na zadanem obsahu a vrati true, zda vyrazu vyhovuje zadany pattern.
        /// </summary>
        /// <param name="pattern">regularni vyraz</param>
        /// <param name="content">obsah, na kterem se vykona regularni vyraz</param>
        /// <returns>true, pokud v obsahu byl nalezen zadany pattern</returns>
        public static bool RegexMatch(string pattern, string content)
        {
            GroupCollection g;
            return RegexMatch(pattern, content, out g);
        }
        /// <summary>
        /// Na zadanem obsahu nahradi text vyhovujici regularnimu vyrazu obsahem v replace. V replace lze pouzit capture groups prostrednictvim $x, kde x je cislo capture group.
        /// </summary>
        /// <param name="pattern">regularni vyraz</param>
        /// <param name="content">obsah, na kterem se vykona regularni vyraz</param>
        /// <param name="replace">text, ktery kterym bude nahrazen text vyhovujici patternu</param>
        /// <returns>vysledny text po nahrazeni</returns>
        public static string RegexReplace(string pattern, string content, string replace)
        {
            Regex r = new Regex(pattern);
            return r.Replace(content, replace);
        }
        /// <summary>
        /// Provede regularni vyraz na zadanem obsahu a vrati true, zda vyrazu vyhovuje zadany pattern. Pokud bylo nalezeno vyce shod, tak vsechny shody jsou vraceny v parametru matches.
        /// </summary>
        /// <param name="pattern">regularni vyraz</param>
        /// <param name="content">obsah, na kterem se vykona regularni vyraz</param>
        /// <param name="matches">Kolekce vsech nalezenych shod</param>
        /// <returns>true, pokud v obsahu byl nalezen alespon jednou zadany pattern</returns>
        public static bool RegexMatchAll(string pattern, string content, out MatchCollection matches)
        {
            Regex r = new Regex(pattern);
            MatchCollection m = r.Matches(content);
            if (m.Count == 0)
            {
                matches = null;
                return false;
            }

            matches = m;
            return true;
        }
    }
}